//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <SI_EFM8SB1_Register_Enums.h>                  // SFR declarations
#include "Project_DefsVarsFuncs.h"
#include "InitDevice.h"
#include "cslib_config.h"
#include "cslib.h"

bit DISPLAY_ANGLE_OR_DELTAS = ANGLE;
bit NEED_TO_CLEAR_SCREEN = FALSE;

//-----------------------------------------------------------------------------
// main() Routine
// ----------------------------------------------------------------------------
int main(void)
{
	unsigned char n;

	unsigned long Accumulated_Capacitance_Sensor1;
	unsigned long Accumulated_Capacitance_Sensor2;
	unsigned long Accumulated_Capacitance_Sensor3;

	unsigned int Sensor1_Unpressed;
	unsigned int Sensor2_Unpressed;
	unsigned int Sensor3_Unpressed;

	unsigned int Sensor1_Measurement;
	unsigned int Sensor2_Measurement;
	unsigned int Sensor3_Measurement;

	int Sensor1_Delta;
	int Sensor2_Delta;
	int Sensor3_Delta;

	unsigned char Segment;

	unsigned int AngularPosition;
	unsigned int TotalDelta;
	float Ratio;

	//call hardware initialization routine
	enter_DefaultMode_from_RESET();

	//give the EFM8 control over the LCD module
	EFM_DISP_ENABLE = HIGH;

	//initialize chip select to LOW (inactive)
	SPI_CS = LOW;

	//clear the LCD screen
	LCD_Clear_All();

	//enable interrupts (we are using only the port-match interrupt)
	IE_EA = TRUE;

	SFRPAGE = CAPSENSE_PAGE;

	//enable the cap-sense module
	CS0CN0_CSEN = TRUE;

	//select 16-bit conversions
	CS0MD2 &= ~(BIT7|BIT6);
	CS0MD2 |= (BIT7|BIT6);

	//make sure that the end-of-measurement interrupt flag is cleared
	CS0CN0_CSINT = CLEARED;

	Accumulated_Capacitance_Sensor1 = 0;
	Accumulated_Capacitance_Sensor2 = 0;
	Accumulated_Capacitance_Sensor3 = 0;

	for (n = 0; n < 16; n++)
	{
		Accumulated_Capacitance_Sensor1 += Measure_Capacitance(SENSOR_1);
		Delay_us(1000);
		Accumulated_Capacitance_Sensor2 += Measure_Capacitance(SENSOR_2);
		Delay_us(1000);
		Accumulated_Capacitance_Sensor3 += Measure_Capacitance(SENSOR_3);
		Delay_10ms(5); Delay_us(6000);
	}

	Sensor1_Unpressed = (Accumulated_Capacitance_Sensor1 >> 4);
	Sensor2_Unpressed = (Accumulated_Capacitance_Sensor2 >> 4);
	Sensor3_Unpressed = (Accumulated_Capacitance_Sensor3 >> 4);

	while(1)
	{
		Sensor1_Measurement = Measure_Capacitance(SENSOR_1);
		Sensor1_Delta = Sensor1_Measurement - Sensor1_Unpressed;
		if(Sensor1_Delta < 0)
			Sensor1_Delta = 0;
		Delay_us(1000);

		Sensor2_Measurement = Measure_Capacitance(SENSOR_2);
		Sensor2_Delta = Sensor2_Measurement - Sensor2_Unpressed;
		if(Sensor2_Delta < 0)
			Sensor2_Delta = 0;
		Delay_us(1000);

		Sensor3_Measurement = Measure_Capacitance(SENSOR_3);
		Sensor3_Delta = Sensor3_Measurement - Sensor3_Unpressed;
		if(Sensor3_Delta < 0)
			Sensor3_Delta = 0;

		if(Sensor1_Delta > TOUCH_DELTA_THRESHOLD
				|| Sensor2_Delta > TOUCH_DELTA_THRESHOLD
				|| Sensor3_Delta > TOUCH_DELTA_THRESHOLD)
		{
			switch(Find_Minimum(Sensor1_Delta, Sensor2_Delta, Sensor3_Delta))
			{
				case SENSOR_1: Segment = UPPER_MIDDLE;
				break;

				case SENSOR_2: Segment = LOWER_RIGHT;
				break;

				case SENSOR_3: Segment = LOWER_LEFT;
				break;
			}

			if(Segment == UPPER_MIDDLE)
			{
				TotalDelta = Sensor2_Delta + Sensor3_Delta;

				Ratio = (float)Sensor2_Delta/TotalDelta;
				Ratio = (Ratio*1.1111) - 0.05556;

				AngularPosition = (Ratio*120) + 30;
			}

			else if(Segment == LOWER_RIGHT)
			{
				TotalDelta = Sensor3_Delta + Sensor1_Delta;

				Ratio = (float)Sensor3_Delta/TotalDelta;
				Ratio = (Ratio*1.1111) - 0.05556;

				AngularPosition = (Ratio*120) + 270;

				if(AngularPosition >= 360)
					AngularPosition = AngularPosition - 360;
			}

			else if(Segment == LOWER_LEFT)
			{
				TotalDelta = Sensor2_Delta + Sensor1_Delta;

				Ratio = (float)Sensor1_Delta/TotalDelta;
				Ratio = (Ratio*1.1111) - 0.05556;

				AngularPosition = (Ratio*120) + 150;
			}
		}

		if(NEED_TO_CLEAR_SCREEN == TRUE)
		{
			NEED_TO_CLEAR_SCREEN = FALSE;
			LCD_Clear_All();
		}

		if(DISPLAY_ANGLE_OR_DELTAS == ANGLE)
		{
			//print the angular position on the LCD
			Update_LCD(ANGLE_DISP, AngularPosition);
		}

		else
		{
			//print the measured value on the LCD
			Update_LCD(SENSOR_1, Sensor1_Delta);

			//print the measured value on the LCD
			Update_LCD(SENSOR_2, Sensor2_Delta);

			//print the measured value on the LCD
			Update_LCD(SENSOR_3, Sensor3_Delta);
		}

		Delay_10ms(5);
	}
}

unsigned int Measure_Capacitance(unsigned char SensorNumber)
{
	//select channel:
	//SENSOR_1 = channel 2 = P0.2 = bottom-middle
	//SENSOR_2 = channel 3 = P0.3 = top-left
	//SENSOR_3 = channel 13 = P1.5 = top-right
	switch(SensorNumber)
	{
		case SENSOR_1: CS0MX = SENSOR_1_CHANNEL;
		break;

		case SENSOR_2: CS0MX = SENSOR_2_CHANNEL;
		break;

		case SENSOR_3: CS0MX = SENSOR_3_CHANNEL;
		break;
	}

	//initiate a capacitance measurement
	CS0CN0_CSBUSY = TRUE;
	while(!CS0CN0_CSINT); CS0CN0_CSINT = CLEARED;

	return CS0D;
}

unsigned char Find_Minimum(unsigned int Sensor1, unsigned int Sensor2, unsigned int Sensor3)
{
	if(Sensor1 < Sensor2 && Sensor1 < Sensor3)
		return SENSOR_1;

	else if(Sensor2 < Sensor1 && Sensor2 < Sensor3)
		return SENSOR_2;

	else if(Sensor3 < Sensor2 && Sensor3 < Sensor1)
		return SENSOR_3;

	else
		return 0;
}
