#include <SI_EFM8SB1_Register_Enums.h>		// SFR declarations
#include "Project_DefsVarsFuncs.h"

void LCD_Clear_All()
{
	SPI_CS = HIGH;	//set chip select HIGH to enable data transfer to the LCD

	Delay_us(10);	//this ensures adequate delay from driving chip select HIGH to the start of data transfer (tsSCS in the LCD data sheet)

	SFRPAGE = SPI0_PAGE;

	SPI0DAT = LCDMODE_ALLCLEAR;	//write the mode-select byte
	while(!SPI0CN0_SPIF); SPI0CN0_SPIF = CLEARED;

	Write_Two_Dummy_Bytes();

	Delay_us(10);	//this ensures adequate delay from the end of data transfer to driving chip select LOW (thSCS in the LCD data sheet)

	SPI_CS = LOW;	//set chip select LOW
}

void Update_LCD(unsigned char SensorNumber, unsigned long DisplayValue)
{
	unsigned char line, column_byte, number_of_lines_sent;
	unsigned char first_line;
	unsigned char remainder;
	unsigned char *FirstDigit, *SecondDigit, *ThirdDigit, *FourthDigit, *FifthDigit;

	//this function cannot process numbers longer than 5 digits
	if(DisplayValue > 99999)
		return;

	/*The modulus operator is used to obtain the fifth digit, which
	 corresponds to the remainder that would result from dividing
	 by 10; we then divide the measured value by 10 and repeat
	 the modulus operation to obtain the remainders corresponding
	 to the other four digits.*/
	remainder = DisplayValue % 10;
	FifthDigit = Match_Digit_to_Array(remainder);

	DisplayValue = DisplayValue/10;
	remainder = DisplayValue % 10;
	FourthDigit = Match_Digit_to_Array(remainder);

	DisplayValue = DisplayValue/10;
	remainder = DisplayValue % 10;
	ThirdDigit = Match_Digit_to_Array(remainder);

	DisplayValue = DisplayValue/10;
	remainder = DisplayValue % 10;
	SecondDigit = Match_Digit_to_Array(remainder);

	DisplayValue = DisplayValue/10;
	remainder = DisplayValue % 10;
	FirstDigit = Match_Digit_to_Array(remainder);

	SPI_CS = HIGH;	//set chip select HIGH to enable data transfer to the LCD

	Delay_us(10);	//this ensures adequate delay from driving chip select HIGH to the start of data transfer (tsSCS in the LCD data sheet)

	SFRPAGE = SPI0_PAGE;

	SPI0DAT = LCDMODE_DATAUPDATE;	//write the mode-select byte
	while(!SPI0CN0_SPIF); SPI0CN0_SPIF = CLEARED;

	switch(SensorNumber)
	{
		case SENSOR_1: first_line = 1;
		break;

		case SENSOR_2: first_line = (128/2) - (DIGIT_HEIGHT_PIX/2);
		break;

		case SENSOR_3: first_line = 127 - DIGIT_HEIGHT_PIX;
		break;

		case ANGLE_DISP: first_line = (128/2) - (DIGIT_HEIGHT_PIX/2);
		break;
	}

	number_of_lines_sent = 0;
	for(line = first_line; line < (first_line + DIGIT_HEIGHT_PIX); line++)
	{
		//send the line address using the lookup table
		SPI0DAT = LCDLineAddr_LUT[line];
		while(!SPI0CN0_SPIF); SPI0CN0_SPIF = CLEARED;

		//send the pixel data for the relevant line of pixels in the display character corresponding to the first digit
		for(column_byte = 0; column_byte < DIGIT_WIDTH_BYTE; column_byte++)
		{
			SPI0DAT = FirstDigit[(number_of_lines_sent*DIGIT_WIDTH_BYTE)  + column_byte];
			while(!SPI0CN0_SPIF); SPI0CN0_SPIF = CLEARED;
		}

		//send the pixel data for the relevant line of pixels in the display character corresponding to the second digit
		for(column_byte = 0; column_byte < DIGIT_WIDTH_BYTE; column_byte++)
		{
			SPI0DAT = SecondDigit[(number_of_lines_sent*DIGIT_WIDTH_BYTE)  + column_byte];
			while(!SPI0CN0_SPIF); SPI0CN0_SPIF = CLEARED;
		}

		//send the pixel data for the relevant line of pixels in the display character corresponding to the third digit
		for(column_byte = 0; column_byte < DIGIT_WIDTH_BYTE; column_byte++)
		{
			SPI0DAT = ThirdDigit[(number_of_lines_sent*DIGIT_WIDTH_BYTE)  + column_byte];
			while(!SPI0CN0_SPIF); SPI0CN0_SPIF = CLEARED;
		}

		//send the pixel data for the relevant line of pixels in the display character corresponding to the fourth digit
		for(column_byte = 0; column_byte < DIGIT_WIDTH_BYTE; column_byte++)
		{
			SPI0DAT = FourthDigit[(number_of_lines_sent*DIGIT_WIDTH_BYTE)  + column_byte];
			while(!SPI0CN0_SPIF); SPI0CN0_SPIF = CLEARED;
		}

		//send the pixel data for the relevant line of pixels in the display character corresponding to the fifth digit
		for(column_byte = 0; column_byte < DIGIT_WIDTH_BYTE; column_byte++)
		{
			SPI0DAT = FifthDigit[(number_of_lines_sent*DIGIT_WIDTH_BYTE)  + column_byte];
			while(!SPI0CN0_SPIF); SPI0CN0_SPIF = CLEARED;
		}

		//send one last byte to finish off the line (so far we have sent only 120 of 128 pixels)
		SPI0DAT = 0xFF;
		while(!SPI0CN0_SPIF); SPI0CN0_SPIF = CLEARED;

		number_of_lines_sent++;

		Write_One_Dummy_Byte();
	}

	Write_Two_Dummy_Bytes();

	Delay_us(10);	//this ensures adequate delay from the end of data transfer to driving chip select LOW (thSCS in the LCD data sheet)

	SPI_CS = LOW;	//set chip select LOW
}

//this function assumes that the SFR page is already set to the SPI page
void Write_One_Dummy_Byte(void)
{
	SPI0DAT = 0x00;	//write dummy byte
	while(!SPI0CN0_SPIF); SPI0CN0_SPIF = CLEARED;
}

//this function assumes that the SFR page is already set to the SPI page
void Write_Two_Dummy_Bytes(void)
{
	SPI0DAT = 0x00;	//write first dummy byte
	while(!SPI0CN0_SPIF); SPI0CN0_SPIF = CLEARED;

	SPI0DAT = 0x00;	//write second dummy byte
	while(!SPI0CN0_SPIF); SPI0CN0_SPIF = CLEARED;
}

/*This function returns a pointer to the pixel data array
that corresponds to the value in InputDigit.*/
unsigned char* Match_Digit_to_Array(unsigned char InputDigit)
{
	switch(InputDigit)
	{
		case 0: return LCD_Char0;
		break;

		case 1: return LCD_Char1;
		break;

		case 2: return LCD_Char2;
		break;

		case 3: return LCD_Char3;
		break;

		case 4: return LCD_Char4;
		break;

		case 5: return LCD_Char5;
		break;

		case 6: return LCD_Char6;
		break;

		case 7: return LCD_Char7;
		break;

		case 8: return LCD_Char8;
		break;

		case 9: return LCD_Char9;
		break;
	}

	/*If for some reason InputDigit is greater than 9,
	 we display an "X" character to indicate an error.*/
	return LCD_Error;
}
